/***************************************************************************
                          kblticker.cpp  -  description
                             -------------------
    begin                : Mon Feb 16 10:46:56 MSK 2004
    copyright            : (C) 2004 by Konstantin Grusha
    email                : zoi@project7.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kblticker.h"

KBLTicker::KBLTicker()
{
	m_kbconf.load(config());

	m_trayicon = new KBLTickerTrayIcon(&m_kbconf);
	setMainWidget(m_trayicon);
	m_trayicon->show();

	// Set up connections
	connect(&bl_pinger, SIGNAL(done(bool)), this, SLOT(slotPingDone(bool)));
	connect(this, SIGNAL(pingIt()), this, SLOT(slotPingBlogLines()));
	connect(this, SIGNAL(raiseMeFlag(int)), m_trayicon, SLOT(slotRaiseFlag(int)));
	connect(this, SIGNAL(dropMeFlag()), m_trayicon, SLOT(slotDropFlag()));
	connect(this, SIGNAL(haveUpdatedUrl(QString*)), this, SLOT(slotGetBrowser(QString*)));
	connect(m_trayicon, SIGNAL(clicked()), this, SLOT(slotLaunchBrowser()));
	connect(m_trayicon, SIGNAL(preferencesSelected()), this, SLOT(slotPreferences()));

	// First time? Force user to enter his email
	if (m_kbconf.email() == "joe@random.hacker") {
		cout << "Our first time!" << endl;
		KMessageBox::information(m_trayicon, i18n("This is the first time you've run KBLTicker. You must specify your e-mail address registered with BlogLines to receive notifications of new feeds on your account.\n\nA configuration dialog will now appear for your convenience."), "Initial settings", QString::null, KMessageBox::Notify);
		slotPreferences();
	}

	// Set up timers
	QTimer *t = new QTimer(this);
	int ping_next = m_kbconf.check_period() * 60000;      // 60 000 == one minute
	connect(t, SIGNAL(timeout()), SLOT(slotPingBlogLines()));
	t->start(ping_next, FALSE);

	// We don't yet have the updates url
	url_updates = "";

	cout << "I'm alive" << endl;
}

KBLTicker::~KBLTicker()
{
	m_kbconf.save(config());
	cout << "Goodbye" << endl;
}

/** Pings BlogLines.com with your data. */
void KBLTicker::slotPingBlogLines(){
	m_trayicon->updateTrayTooltip(i18n("Checking for new feeds..."));

	QString request = "/update?user=" + m_kbconf.email() + "&ver=1";
	QString host = "rpc.bloglines.com";
	QString client = "BlogLines Client/Linux/KDE";

	QHttpRequestHeader header("GET", request);
	header.setValue("Host", host);
	header.setValue("User-Agent", client);

	bl_pinger.setHost(host);
	bl_pinger.request(header);

	cout << "Ping!" << endl;
}

/** Called upon arrival of new data. */
void KBLTicker::slotPingDone(bool error){
	if (error) {
		QString tmp_report = i18n("Unable to check for updates:\n%1").arg(bl_pinger.errorString());
		m_trayicon->updateTrayTooltip(tmp_report);
		cerr << "*** " << tmp_report << endl;
	} else {
		QStringList ping_results = QStringList::split("|", bl_pinger.readAll());   // [0] holds the number of entries, [1] holds the optional update URL
		int new_entries = ping_results[0].toInt();
		if (new_entries > 0) {
			cout << "New entries: " << new_entries << endl;
			emit raiseMeFlag(new_entries);

			url_updates="";
			QRegExp rx("^http.*$");

			if (rx.search(ping_results[1]) != -1) {
				QString new_url=ping_results[1];
				cout << "I also have this URL: " << new_url << endl;
				emit haveUpdatedUrl(&new_url);
			} else {
				QString new_url="http://www.bloglines.com/myblogs";
				cout << "Falling back to default URL " << new_url << endl;
				emit haveUpdatedUrl(&new_url);
			}

		} else if (new_entries == -1) {
			QString tmp_report = i18n("Unable to check for updates:\n%1").arg(i18n("Invalid e-mail address"));
			m_trayicon->updateTrayTooltip(tmp_report);
			cerr << "*** " << tmp_report << endl;
		} else if (new_entries == 0) {
			QString tmp_report = i18n("No new entries");
			m_trayicon->updateTrayTooltip(tmp_report);
			cout << "No new entries" << endl;			// Do nothing
		}
		cout << "Pong!" << endl;
	}
}

/** Prepare to launch konqueror upon left-click on the icon. */
void KBLTicker::slotGetBrowser(QString *url){
	QString tmp = *url;
	cout << "I'll point your browser at " << tmp << endl;
	url_updates = tmp;
}

/** Brings up konqueror with the url returned from the search or forces ping. */
void KBLTicker::slotLaunchBrowser(){
	if (m_trayicon->flagRaised() && url_updates!="") {
		cout << "Firing up konqueror and pointing it to " << url_updates << endl;

		QProcess proc;
		proc.addArgument("konqueror");
		proc.addArgument(url_updates);

		if (!proc.start()) {
			cerr << "*** Can't launch konqueror?!" << endl;
		} else {
			emit dropMeFlag();
		}
	}
	else if (m_trayicon->flagRaised() && url_updates=="") {
		cout << "I wish I could bring you the news, but I don't know the URL" << endl;
		// TODO: Remove? Not sure. This should never occur since we fallback to default if no url received.
	}
	else if (!m_trayicon->flagRaised()) {
		emit pingIt();
	}
}

void KBLTicker::slotPreferences()
{
  KBLTickerConfigDlg dlg(&m_kbconf);
  dlg.exec();
}
