/***************************************************************************
                          kbltickerconfigdlg  -  description
                             -------------------
    begin                : Sat Feb 21 2004
    copyright            : (C) 2004 by Konstantin Grusha
    email                : zoi@project7.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlayout.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qiconset.h>
#include <qvbox.h>
#include <qstyle.h>
#include <qframe.h>
#include <qgroupbox.h>
#include <qheader.h>
#include <qobjectlist.h>
#include <qgroupbox.h>
#include <qwhatsthis.h>

#include <kfiledialog.h>
#include <kstandarddirs.h>
#include <kdialogbase.h>
#include <kpushbutton.h>
#include <kdeversion.h>
#include <kseparator.h>
#include <klistbox.h>
#include <klocale.h>
#include <kdebug.h>
#if KDE_VERSION_MAJOR >= 3
  #include <kapplication.h>
#else
  #include <kapp.h>
#endif
#include <kurl.h>

#include "kbltickerconfigdlg.h"

KBLTickerConfigDlg::KBLTickerConfigDlg(KBLTickerConfig *kbconf, QWidget *parent, const char *name) : KDialogBase(Swallow, i18n("Configure")/*caption*/, Ok | Cancel, Ok, parent, name, true /*modal*/)
{
	m_kbconf = kbconf;

	setButtonBoxOrientation(Horizontal);
	setupGeneralPage();
	showConfig();
	slotToggleBellsWhistles();
}

void KBLTickerConfigDlg::setupGeneralPage()
{
	QWidget *page = new QWidget(this);
	setMainWidget(page);
	QVBoxLayout *vlayout = new QVBoxLayout(page, 0, spacingHint());
	vlayout->setSpacing(5);

	// Notification settings box
	QGroupBox *notif = new QGroupBox(i18n("Notification Settings"), page);
	QVBoxLayout *notif_layout = new QVBoxLayout(notif);
	notif_layout->setSpacing(5);
	notif_layout->setMargin(15);

	chkToggleMessage = new QCheckBox(i18n("Use message balloon notification"), notif);
	notif_layout->addWidget(chkToggleMessage);
	QWhatsThis::add(chkToggleMessage, i18n("When this checkbox is checked, a little balloon containing the number of new entries on your BlogLines account will appear near your system tray icon. It will automagically disappear after three seconds."));

	chkToggleSound = new QCheckBox(i18n("Use sound notification"), notif);
	notif_layout->addWidget(chkToggleSound);
	QWhatsThis::add(chkToggleSound, i18n("When this checkbox is checked, you will hear a sound every time you have new entries on your BlogLines account."));
	connect(chkToggleSound, SIGNAL(clicked()), this, SLOT(slotToggleBellsWhistles()));

	// Bells
	QHBoxLayout *bells_layout = new QHBoxLayout(notif_layout);
	QLabel *lblBells = new QLabel(notif);
	lblBells->setText(i18n("Bells:"));
	edtBells = new KLineEdit(notif);
	btnBells = new KPushButton("...", notif);
	QWhatsThis::add(lblBells, i18n("You will hear bells when the number of new entries on your account is below a specified threshold. If you are hearing them, you may probably want to check out your aggregation page, for there are some news -- just not too many yet."));
	QWhatsThis::add(edtBells, i18n("You will hear bells when the number of new entries on your account is below a specified threshold. If you are hearing them, you may probably want to check out your aggregation page, for there are some news -- just not too many yet."));

	bells_layout->addWidget(lblBells);
	bells_layout->addWidget(edtBells);
	bells_layout->addWidget(btnBells);
	connect(btnBells, SIGNAL(clicked()), this, SLOT(slotSelectBells()));

	// Whistles
	QHBoxLayout *whistles_layout = new QHBoxLayout(notif_layout);
	QLabel *lblWhistles = new QLabel(notif);
	lblWhistles->setText(i18n("Whistles:"));
	edtWhistles = new KLineEdit(notif);
	btnWhistles = new KPushButton("...", notif);
	QWhatsThis::add(lblWhistles, i18n("You will hear whistles when the number of new entries on your account will reach a specified threshold. If you are hearing them, you should probably check out your aggregation page, for there will be enough news for you to read."));
	QWhatsThis::add(edtWhistles, i18n("You will hear whistles when the number of new entries on your account will reach a specified threshold. If you are hearing them, you should probably check out your aggregation page, for there will be enough news for you to read."));

	whistles_layout->addWidget(lblWhistles);
	whistles_layout->addWidget(edtWhistles);
	whistles_layout->addWidget(btnWhistles);
	connect(btnWhistles, SIGNAL(clicked()), this, SLOT(slotSelectWhistles()));

	// Threshold
	QHBoxLayout *threshold_layout = new QHBoxLayout(notif_layout);
	QLabel *lblThresholdPrefix = new QLabel(notif);
	lblThresholdPrefix->setText(i18n("Bells/Whistles Threshold:"));
	QLabel *lblThresholdSuffix = new QLabel(notif);
	lblThresholdSuffix->setText(i18n("new entries"));
	spnThreshold = new QSpinBox(1, 100, 1, notif);	// 100 is a sane number of new entries, one'll hardly need more?
	QWhatsThis::add(spnThreshold, i18n("Threshold affects sound notification. Set it to 1 if you don't care about the number of new entries and want to read them even if there were just one new entry. Default is ten."));
	QWhatsThis::add(lblThresholdPrefix, i18n("Threshold affects sound notification. Set it to 1 if you don't care about the number of new entries and want to read them even if there were just one new entry. Default is ten."));
	QWhatsThis::add(lblThresholdSuffix, i18n("Threshold affects sound notification. Set it to 1 if you don't care about the number of new entries and want to read them even if there were just one new entry. Default is ten."));

	threshold_layout->addWidget(lblThresholdPrefix);
	threshold_layout->addWidget(spnThreshold);
	threshold_layout->addWidget(lblThresholdSuffix);


	vlayout->addWidget(notif);
	// End of notification box

	// Checking settings box
  QGroupBox *check = new QGroupBox(i18n("Checking Settings"), page);
	QVBoxLayout *check_layout = new QVBoxLayout(check);
	check_layout->setSpacing(5);
	check_layout->setMargin(15);

	QHBoxLayout *spin_layout = new QHBoxLayout(check_layout);

	QLabel *lblCheckPrefix = new QLabel(check);
	lblCheckPrefix->setText(i18n("Check every"));
	QLabel *lblCheckSuffix = new QLabel(check);
	lblCheckSuffix->setText(i18n("minutes"));
	spnCheckPeriod = new QSpinBox(1, 1440, 1, check);		// Allow min period of 1m and max of 24h
	QWhatsThis::add(spnCheckPeriod, i18n("Your BlogLines account will be checked in periods specified there."));
	QWhatsThis::add(lblCheckPrefix, i18n("Your BlogLines account will be checked in periods specified there."));
	QWhatsThis::add(lblCheckSuffix, i18n("Your BlogLines account will be checked in periods specified there."));

	spin_layout->addWidget(lblCheckPrefix);
	spin_layout->addWidget(spnCheckPeriod);
	spin_layout->addWidget(lblCheckSuffix);

	QLabel *lblEmail = new QLabel(check);
	lblEmail->setText(i18n("BlogLines registered e-mail:"));
	check_layout->addWidget(lblEmail);
	edtEmail = new KLineEdit(check);
	check_layout->addWidget(edtEmail);

	vlayout->addWidget(check);
	// End of checking settings box
}

void KBLTickerConfigDlg::showConfig()
{
	chkToggleSound->setChecked(m_kbconf->notify_sound_on());
	chkToggleMessage->setChecked(m_kbconf->notify_message_on());
	spnCheckPeriod->setValue(m_kbconf->check_period());
	spnThreshold->setValue(m_kbconf->threshold());
	edtBells->setText(m_kbconf->notify_bells());
	edtWhistles->setText(m_kbconf->notify_whistles());
	edtEmail->setText(m_kbconf->email());
}


KBLTickerConfigDlg::~KBLTickerConfigDlg()
{
}

void KBLTickerConfigDlg::slotOk()
{
	saveConfig();
	KDialogBase::slotOk();
}


void KBLTickerConfigDlg::slotSelectBells()
{
	KURL *tmpurl = new KURL(m_kbconf->notify_bells());
	cout << "Select bells from " << tmpurl->directory() << endl;

	QString file_name = KFileDialog::getOpenFileName(tmpurl->directory(), "Sounds (*.ogg *.wav *.mp3)", this, i18n("Select sound for bells") );

	if (!file_name.isEmpty()) {
		edtBells->setText(file_name);
		cout << "I got this: " << file_name << endl;
	}
}

void KBLTickerConfigDlg::slotSelectWhistles()
{
	KURL *tmpurl = new KURL(m_kbconf->notify_whistles());
	cout << "Select whistles from " << tmpurl->directory() << endl;

	QString file_name = KFileDialog::getOpenFileName(tmpurl->directory(), "Sounds (*.ogg *.wav *.mp3)", this, i18n("Select sound for whistles") );

	if (!file_name.isEmpty()) {
		edtWhistles->setText(file_name);
		cout << "I got this: " << file_name << endl;
	}
}

void KBLTickerConfigDlg::slotToggleBellsWhistles()
{
	if (!chkToggleSound->isChecked()) {
		cout << "Disabling bells and whistles" << endl;
		edtBells->setDisabled(true);
		btnBells->setDisabled(true);
		edtWhistles->setDisabled(true);
		btnWhistles->setDisabled(true);
		spnThreshold->setDisabled(true);
	} else {
		cout << "Enabling bells and whistles" << endl;
		edtBells->setEnabled(true);
		btnBells->setEnabled(true);
		edtWhistles->setEnabled(true);
		btnWhistles->setEnabled(true);
		spnThreshold->setEnabled(true);
	}
}

void KBLTickerConfigDlg::saveConfig()
{
	KConfig *config = kapp->config();

	config->setGroup(OPTIONS_SECTION);
	m_kbconf->set_check_period(spnCheckPeriod->value());
	m_kbconf->set_email(edtEmail->displayText());
	m_kbconf->set_threshold(spnThreshold->value());
	m_kbconf->set_notify_bells(edtBells->displayText());
	m_kbconf->set_notify_whistles(edtWhistles->displayText());
	m_kbconf->set_notify_sound_on(chkToggleSound->isChecked());
	m_kbconf->set_notify_message_on(chkToggleMessage->isChecked());
	m_kbconf->save(config);

	config->sync();
}
