/***************************************************************************
                          kbltickertrayicon.cpp  -  description
                             -------------------
    begin                : Mon Feb 16 2004
    copyright            : (C) 2004 by Konstantin Grusha
    email                : zoi@project7.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qtooltip.h>
#include <qstyle.h>
#include <qpainter.h>
#include <qtimer.h>

#include <kdeversion.h>
#include "kbltickertrayicon.h"
#include <kpopupmenu.h>
#if KDE_VERSION_MAJOR >= 3
  #include <kstandarddirs.h>
#else
  #include <kstddirs.h>
#endif
#include <klocale.h>
#include <kstdaction.h>
#include <kaction.h>
#include <kaboutapplication.h>
#if KDE_VERSION_MAJOR >= 3
  #include <kapplication.h>
#else
  #include <kapp.h>
#endif
#include <kiconloader.h>
#include <kdebug.h>

KBLTickerTrayIcon::KBLTickerTrayIcon(KBLTickerConfig *conf){

	m_kbconf = conf;

#if KDE_VERSION_MAJOR >= 3
        KActionCollection *actions = new KActionCollection(this);
        #define ACTION_PARENT actions
#else
        #define ACTION_PARENT this
#endif

	KPopupMenu * menu = contextMenu();

	KAction *pref = KStdAction::preferences(this, SIGNAL(preferencesSelected()), ACTION_PARENT);
	pref->plug(menu);

	KAction *about = KStdAction::aboutApp(this, SLOT(slotAbout()), ACTION_PARENT);
	about->plug(menu);

	cout << "We have a menu" << endl;

	// Set up the icon
	QPixmap pix;
	pix = kapp->iconLoader()->loadIcon("kblticker", KIcon::Small);
	menu->changeTitle(menu->idAt(0), pix, i18n("BlogLines Ticker"));

	updateTrayIcon(0);
	cout << "We have an icon" << endl;

	flag_raised=false;
}

KBLTickerTrayIcon::~KBLTickerTrayIcon(){

}

/** Display the About dialog box */
void KBLTickerTrayIcon::slotAbout(){
	KAboutApplication about;
	about.exec();
}

/** Pull up a flag if there's something new, or pull it down, if there's not. */
void KBLTickerTrayIcon::updateTrayIcon(int status){
	QPixmap pix;

	switch (status) {
		case 0:
			pix = kapp->iconLoader()->loadIcon("kblticker_idle", KIcon::Small);
			setPixmap(pix);
			updateTrayTooltip(i18n("No new entries"));
			flag_raised=false;
			break;
		default:
			pix = kapp->iconLoader()->loadIcon("kblticker_new", KIcon::Small);
			setPixmap(pix);
			
			QString tmp_report;
			if (status==1) {
			    tmp_report = i18n("There is %1 new entry").arg(status);
			} else {
			    tmp_report = i18n("There are %1 new entries").arg(status);
			}
			    
			updateTrayTooltip(tmp_report);
			flag_raised=true;
			break;
	}
}

/** Handle incoming entries */
void KBLTickerTrayIcon::slotRaiseFlag(int new_entries){
	updateTrayIcon(new_entries);

	// Display the balloon
	if (m_kbconf->notify_message_on()) {
		QString tmp_report;
		if (new_entries==1) {
		    tmp_report = i18n("%1 new entry").arg(new_entries);
		} else {
		    tmp_report = i18n("%1 new entries").arg(new_entries);
		}

		m_balloon = new KBLTickerBalloon(tmp_report);
		m_balloon->setAnchor(mapToGlobal(pos()));
		m_balloon->show();

		// Remove in 3 seconds
		QTimer *remove_timer = new QTimer(this);
		int remove_balloon = 3000;
		remove_timer->start(remove_balloon, TRUE);

		connect(remove_timer, SIGNAL(timeout()), this, SLOT(slotRemoveBalloon()));
	}

	// Play the notification
	if (m_kbconf->notify_sound_on())
		if (new_entries < m_kbconf->threshold())
			KAudioPlayer::play(m_kbconf->notify_bells());
		else
			KAudioPlayer::play(m_kbconf->notify_whistles());

	cout << "Raising the flag" << endl;
}

/** Handle no incoming entries */
void KBLTickerTrayIcon::slotDropFlag(){
	updateTrayIcon(0);
	slotRemoveBalloon();
	cout << "Dropping the flag" << endl;
}

/** Remove the balloon */
void KBLTickerTrayIcon::slotRemoveBalloon(){
	if (m_kbconf->notify_message_on())		// Sometimes it returned true even if there was no m_balloon
		if (m_balloon->isVisible())		// and then everything crashed, hence such check
			delete m_balloon;
}

/** No descriptions */
void KBLTickerTrayIcon::mouseReleaseEvent(QMouseEvent *event){
	if (event->button() == LeftButton) {
		emit clicked();
	}
}
